#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import string

# import interfaces
from zope.schema.interfaces import IText, ITextLine, IDecimal, IList, IDict, ITuple, IPassword

# import Zope3 packages
from persistent.list import PersistentList as PersistentListType
from persistent.mapping import PersistentMapping
from zope.interface import implementer
from zope.schema import Text, TextLine, Decimal, List, Dict, Tuple, Password, ValidationError

# import local packages

from pyams_utils import _


#
# Persistent list field
#

class IPersistentList(IList):
    """Persistent list field marker interface"""


@implementer(IPersistentList)
class PersistentList(List):
    """Persistent list field"""

    _type = PersistentListType


#
# Persistent mapping field
#

class IPersistentDict(IDict):
    """Persistent mapping field marker interface"""


@implementer(IPersistentDict)
class PersistentDict(Dict):
    """Persistent mapping field"""

    _type = PersistentMapping


#
# Encoded password field
#

class IEncodedPassword(IPassword):
    """Encoded password field interface"""


@implementer(IEncodedPassword)
class EncodedPassword(Password):
    """Encoded password field"""

    _type = None

    def fromUnicode(self, str):
        return str

    def constraint(self, value):
        return True


#
# HTML field
#

class IHTMLField(IText):
    """HTML field interface"""


@implementer(IHTMLField)
class HTMLField(Text):
    """HTML field"""


#
# Color field
#

class IColorField(ITextLine):
    """Marker interface for color fields"""


@implementer(IColorField)
class ColorField(TextLine):
    """Color field"""

    def __init__(self, *args, **kw):
        super(ColorField, self).__init__(max_length=6, *args, **kw)

    def _validate(self, value):
        if len(value) not in (3, 6):
            raise ValidationError(_("Color length must be 3 or 6 characters"))
        for v in value:
            if v not in string.hexdigits:
                raise ValidationError(_("Color value must contain only valid hexadecimal color codes (numbers or "
                                        "letters between 'A' end 'F')"))
        super(ColorField, self)._validate(value)


#
# Pointed decimal field
#

class IDottedDecimalField(IDecimal):
    """Marker interface for dotted decimal fields"""


@implementer(IDottedDecimalField)
class DottedDecimalField(Decimal):
    """Dotted decimal field"""


#
# Dates range field
#

class IDatesRangeField(ITuple):
    """Marker interface for dates range fields"""


@implementer(IDatesRangeField)
class DatesRangeField(Tuple):
    """Dates range field"""

    def __init__(self, value_type=None, unique=False, **kw):
        super(DatesRangeField, self).__init__(value_type=None, unique=False,
                                              min_length=2, max_length=2, **kw)


#
# TextLine list field
#

class ITextLineListField(IList):
    """Marker interface for textline list field"""


@implementer(ITextLineListField)
class TextLineListField(List):
    """TextLine list field"""

    def __init__(self, value_type=None, unique=False, **kw):
        super(TextLineListField, self).__init__(value_type=TextLine(), unique=True, **kw)
